/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FXXTABLE_HPP
#define FXXTABLE_HPP


#include <fx.h>

#include <aqfoxext/aqfoxext.h>

#include <list>


class FOX16_HtmlCtx;


/**
 * This class implements a generic table. This class makes no assumption about the content of
 * the table.
 *
 * The following virtual methods MUST be implemented:
 * <ul>
 *  <li>
 *    @ref getColumnInfo: This method should create and return  a ColumnInfo object for the
 *    given row/column. The object returned might contain data which is only needed for
 *    visible cells. Those objects are only used as long as the corresponding cell is visible.
 *  </li>
 *  <li>
 *    @ref releaseColumnInfo: This method is called when a cell is no longer visible and thus the
 *    information is no longer needed.
 *  </li>
 * </ul>
 *
 * If you want the table to be editable you should implement the following virtual methods:
 * <ul>
 *  <li>
 *    @ref getEditor: This method should create and return a widget which can be used for
 *    editing the given cell. The widget is positioned at the exact same position of the
 *    cell on the screen. The object returned shoould have the table as parent.
 *  </li>
 *  <li>
 *    @ref setFromEditor: This method is called when the input is accepted (e.g. when the user
 *    moves to another cell etc). The implementation should store the data from the editor widget.
 *  </li>
 *  <li>
 *    @ref releaseEditor: This method is called as soon as the editing mode is left. The simplest
 *    implementation would just delete the given widget.
 *  </li>
 * </ul>
 *
 * The table sends the signal "SEL_SELECTED" whenever the selection changes. We currently use the
 * same signal for selection and deselection here, maybe this will later be changed.
 * The ptr argument to this signal will be NULL (because this signal doesn't indicate a specific
 * table item, again: may be that will change later).
 *
 */
class AQFOXEXT_API FXXTable: public FXComposite {
  FXDECLARE(FXXTable)

public:
  enum {
    DEFAULT_MARGIN=2,
    DEFAULT_GRIDWIDTH=1,
    DEFAULT_HSPACING=2,
    DEFAULT_VSPACING=2,
    DEFAULT_COLUMN_WIDTH=30,
    NUM_MARK_COLOR=4
  };

  enum {
    TABLE_TRACK_SCROLLBARS = 0x00010000,
    TABLE_TRACK_HEADERS    = 0x00020000,
    TABLE_MARK_FULL_ROW    = 0x00040000,
    TABLE_READONLY         = 0x00080000
  };

  enum {
    COPYMODE_NONE=0,
    COPYMODE_COPY,
    COPYMODE_ATTACH
  };

  enum {
    ID_CHEADER=FXComposite::ID_LAST,
    ID_HSCROLLBAR,
    ID_VSCROLLBAR,
    ID_TABLEITEM,

    ID_MOVE_LEFT,
    ID_MOVE_RIGHT,
    ID_MOVE_UP,
    ID_MOVE_DOWN,
    ID_MOVE_TOP,
    ID_MOVE_BOTTOM,
    ID_MOVE_PAGEDOWN,
    ID_MOVE_PAGEUP,
    ID_START_INPUT,
    ID_CANCEL_INPUT,
    ID_ACCEPT_INPUT,
    ID_MARK,
    ID_EXTEND,
    ID_UNEXTEND,
    ID_SELECT_COLUMN_INDEX,
    ID_SELECT_ROW_INDEX,
    ID_SELECT_COLUMN,
    ID_SELECT_ROW,
    ID_SELECT_CELL,
    ID_SELECT_ALL,
    ID_DESELECT_ALL,


    ID_LAST
  };

  class AQFOXEXT_API TableCoords {
  public:
    int row;
    int column;

    TableCoords(): row(-1), column(-1) {};
    TableCoords(int r, int c): row(r), column(c) {};
    ~TableCoords() {};
  };

  class AQFOXEXT_API ColumnInfo {
  public:
    ColumnInfo(FXXTable *t, int row, int col);
    virtual ~ColumnInfo();

    int getRow() const { return m_row;};
    int getColumn() const { return m_column;};

    int getX() const { return m_x;};
    void setX(int i) { m_x=i;};

    int getWidth() const { return m_width;};
    void setWidth(int i) { m_width=i;};

    /** returns a code for the special color to be used:
     * <ul>
     *   <li>-1: use color from @ref getBackground()</li>
     *   <li>0: don't use a special color (use default background color of the table)</li>
     *   <li>>0: index into a list of colors (retrieved via @ref FXXTable::getMarkColor)</li>
     * </ul>
     */
    int getUseSpecialColor() const { return m_useSpecialColor;};
    void setUseSpecialColor(int i) { m_useSpecialColor=i;};

    FXColor getBackground() const { return m_backGround;};

    virtual int getHeightForWidth(int forWidth);
    virtual void draw(FXDC &dc, int x, int y, int w, int h);
    virtual void update();

  protected:
    FXXTable *m_table;
    int m_row;
    int m_column;
    int m_x;
    int m_width;
    int m_useSpecialColor;
    FXColor m_backGround;
  };


  class AQFOXEXT_API ColumnInfoText: public ColumnInfo {
  public:
    ColumnInfoText(FXXTable *t, int row, int col, const FXString &s,
                   int useSpecialColor=0);
    ColumnInfoText(FXXTable *t, int row, int col,
		   const FXString &s,
		   FXColor bg);
    virtual ~ColumnInfoText();

    const FXString &getText() const { return m_text;};
    void setText(const FXString &s) { m_text=s;};

    virtual int getHeightForWidth(int forWidth);
    virtual void draw(FXDC &dc, int x, int y, int w, int h);
    virtual void update();

  protected:
    FXString m_text;
    FOX16_HtmlCtx *m_htmlCtx;
  };


  class AQFOXEXT_API ColumnInfoIcons: public ColumnInfo {
  public:
    enum {
      MAX_ICONS=8,
      ICON_SPACING=2
    };

  public:
    ColumnInfoIcons(FXXTable *t, int row, int col,
                    int useSpecialColor=0);
    ColumnInfoIcons(FXXTable *t, int row, int col, FXColor backGround);
    virtual ~ColumnInfoIcons();

    /** this method does not take over the given icon */
    void setIcon(int idx, FXIcon *ic);
    void addIcon(FXIcon *ic);

    virtual int getHeightForWidth(int forWidth);
    virtual void draw(FXDC &dc, int x, int y, int w, int h);
    virtual void update();

  protected:
    FXIcon *m_iconPtrs[MAX_ICONS];
    int m_iconCount;
  };


  class AQFOXEXT_API RowInfo {
  public:
    RowInfo(FXXTable *t, int row);
    ~RowInfo();

    int getRow() const { return m_row;};

    int getY() const { return m_y;};
    void setY(int i) { m_y=i;};

    int getHeight() const { return m_height;};
    void setHeight(int h) { m_height=h;};

    void addColumnInfo(ColumnInfo *ci);
    ColumnInfo *getColumnInfo(int c);
    std::list<ColumnInfo*> &getColumnInfos() { return m_columns;};

    void setModified(bool b) { m_modified=b;};
    bool isModified() const { return m_modified;};

  protected:
    FXXTable *m_table;
    int m_row;
    std::list<ColumnInfo*> m_columns;
    int m_y;
    int m_height;
    bool m_modified;
  };


  class AQFOXEXT_API SelectionRange {
  public:
    int first;
    int last;

    SelectionRange *previous;
    SelectionRange *next;

    SelectionRange(int first, int last);
    ~SelectionRange();

  };

  class AQFOXEXT_API Selection {
  public:
    Selection();
    ~Selection();

    bool contains(int v);

    void addRange(int first, int last);
    void delRange(int first, int last);

    void clear();
    bool isEmpty() const { return (m_first==NULL)?true:false;};

    const SelectionRange *getFirstRange() const { return m_first;};

    void dump(FILE *f, int indent) const;

  protected:
    void unlinkRange(SelectionRange *r);
    void appendRange(SelectionRange *r);
    void insertRange(FXXTable::SelectionRange *where,
                     FXXTable::SelectionRange *r);
    void insertRangeBefore(FXXTable::SelectionRange *where,
                           FXXTable::SelectionRange *r);
    void combineRanges();


    SelectionRange *m_first;
    SelectionRange *m_last;
  };


public:
  FXXTable(FXComposite *p, FXObject* tgt=NULL, FXSelector sel=0, FXuint opts=0,
	   FXint x=0, FXint y=0, FXint w=0, FXint h=0,
	   FXint pl=DEFAULT_MARGIN, FXint pr=DEFAULT_MARGIN,
	   FXint pt=DEFAULT_MARGIN, FXint pb=DEFAULT_MARGIN);

  virtual ~FXXTable();

  int getNumberOfRows() const { return m_nrows;};
  virtual void setNumberOfRows(int i);

  int getNumberOfColumns() const { return m_ncolumns; };
  virtual void addColumn(const FXString &s, int size=DEFAULT_COLUMN_WIDTH);

  FXFont *getFont() const { return m_font;};

  FXColor getCellBackColor(int r, int c);
  void setCellBackColor(int r, int c, FXColor color);
  FXColor getTextColor() const { return m_textColor;};

  FXColor getSelBackColor() const { return m_selbackColor;};
  FXColor getSelTextColor() const { return m_seltextColor;};

  FXColor getMarkColor(int idx) const;
  void setMarkColor(int idx, FXColor c);

  FXColor getMarkColor(int idx, int r, int c) const;
  void setMarkColor(int idx, FXColor col, int r, int c);

  int getRowAtY(int y) const;
  int getColumnAtX(int x) const;

  void setCurrentItem(int r, int c, bool notify=true);
  void makePositionVisible(int r, int c);
  bool isPositionVisible(int r, int c) const;

  int getFirstVisibleRow() const { return m_firstVisibleRow;};
  int getLastVisibleRow() const { return m_lastVisibleRow;};

  int getCurrentRow() const { return m_currentRow;};
  int getCurrentColumn() const { return m_currentCol;};

  bool isEditable() const;

  bool getShowGrid() const { return m_showGrid;};
  void setShowGrid(bool b) { m_showGrid=b;};

  int getGridWidth() const { return m_gridWidth;};
  void setGridWidth(int i) { m_gridWidth=i;};

  void startInput(int r, int c);
  void acceptInput(bool notify=true);
  void cancelInput(bool notify=true);

  void updateCell(int r, int c);
  void updateRow(int r);

  const Selection &getSelection() const { return m_selection;};

  bool isRowSelected(int r);

  int getHeaderSize(int index) const;
  void setHeaderSize(int idx, int size);

  FXbool getHeaderArrowDir(int idx);
  void setHeaderArrowDir(int idx, FXbool b);

  int getNumHeaders() const;

  virtual void sortByColumn(int idx, bool up);

  void sort();

  long onPaint(FXObject*, FXSelector, void*);
  long onHScrollerChanged(FXObject*, FXSelector, void* ptr);
  long onHScrollerDragged(FXObject*, FXSelector, void* ptr);
  long onVScrollerChanged(FXObject*, FXSelector, void* ptr);
  long onVScrollerDragged(FXObject*, FXSelector, void* ptr);

  long onHeaderItemClicked(FXObject*, FXSelector, void* ptr);
  long onHeaderItemDragged(FXObject*, FXSelector, void* ptr);

  long onFocusIn(FXObject*, FXSelector, void*);
  long onFocusOut(FXObject*, FXSelector, void*);

  long onKeyPress(FXObject*, FXSelector, void*);
  long onKeyRelease(FXObject*, FXSelector, void*);

  long onLeftBtnPress(FXObject*, FXSelector, void*);
  long onLeftBtnRelease(FXObject*, FXSelector, void*);

  long onRightBtnPress(FXObject*, FXSelector, void*);
  long onRightBtnRelease(FXObject*, FXSelector, void*);

  long onDoubleClicked(FXObject*, FXSelector, void*);

  long onMouseWheel(FXObject *sender, FXSelector sel, void *ptr);

  long onCmdMoveRight(FXObject*,FXSelector,void*);
  long onCmdMoveLeft(FXObject*,FXSelector,void*);
  long onCmdMoveUp(FXObject*,FXSelector,void*);
  long onCmdMoveDown(FXObject*,FXSelector,void*);
  long onCmdMoveTop(FXObject*,FXSelector,void*);
  long onCmdMoveBottom(FXObject*,FXSelector,void*);
  long onCmdMovePageDown(FXObject*,FXSelector,void*);
  long onCmdMovePageUp(FXObject*,FXSelector,void*);

  long onCmdMark(FXObject*,FXSelector,void*);
  long onCmdExtend(FXObject*,FXSelector,void*);
  long onCmdUnextend(FXObject*, FXSelector, void*);

  long onCmdStartInput(FXObject*, FXSelector, void*);

  long onCmdDeselectAll(FXObject*, FXSelector, void*);

  long onEditorKeyPress(FXObject *sender, FXSelector sel, void* ptr);
  long onEditorKeyRelease(FXObject* sender, FXSelector sel, void* ptr);

  virtual void layout();

  FXint getDefaultWidth();
  FXint getDefaultHeight();

  int getMinRowHeight() const { return m_minRowHeight;};
  void setMinRowHeight(int h) { m_minRowHeight=h;};

  virtual void clearItems();

  void create();
  void detach();

  void rebuild();

  virtual bool canFocus() const;
  void setFocus();
  void killFocus();


protected:
  FXXTable();

  virtual ColumnInfo *getColumnInfo(int r, int c);
  virtual void releaseColumnInfo(ColumnInfo *ci);

  virtual FXWindow *getEditor(int r, int c);
  virtual void setFromEditor(ColumnInfo *ci, FXWindow *ed);
  virtual void releaseEditor(int r, int c, FXWindow *ed);

  RowInfo *getRowInfo(int r, bool needCorrectLayout);
  void releaseRowInfo(RowInfo *ri);
  void modifiedRowInfo(int r);
  void releaseInvisibleRowInfos();
  void releaseAllRowInfos();

  void layoutRowInfo(RowInfo *ri);

  void drawContents(FXDC &dc);
  void restrictClipRectangle(FXDC &dc, int x, int y, int w, int h);

  void unselectAll();

  FXHeader *m_columnHeader;
  FXScrollBar *m_horizontalBar;
  FXScrollBar *m_verticalBar;
  FXScrollCorner *m_scrollCorner;

  int m_nrows;
  int m_ncolumns;

  int m_firstVisibleRow;
  int m_lastVisibleRow;

  int m_marginLeft;
  int m_marginRight;
  int m_marginTop;
  int m_marginBottom;

  int m_viewportX;
  int m_viewportY;
  int m_viewportWidth;
  int m_viewportHeight;

  bool m_showGrid;
  int m_gridWidth;
  int m_hSpacing;
  int m_vSpacing;

  bool m_alignTop;

  int m_currentRow;
  int m_currentCol;

  int m_editRow;
  int m_editCol;

  FXWindow *m_editor;

  FXColor m_textColor;              // Normal text color
  FXColor m_baseColor;              // Base color
  FXColor m_hiliteColor;            // Highlight color
  FXColor m_shadowColor;            // Shadow color
  FXColor m_borderColor;            // Border color
  FXColor m_selbackColor;           // Select background color
  FXColor m_seltextColor;           // Select text color
  FXColor m_gridColor;              // Grid line color
  FXColor m_stippleColor;           // Stipple color
  FXColor m_cellBorderColor;        // Cell border color
  FXColor m_cellBackColor[2][2];    // Row/Column even/odd background color
  FXColor m_markColor[NUM_MARK_COLOR][2][2];

  FXFont *m_font;

  int m_minRowHeight;

  bool m_needLayout;
  bool m_hasFocus;

  std::list<RowInfo*> m_rows;
  Selection m_selection;
};



#endif
