/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <aqfoxext/fxx_menucontainer.hpp>
#include <aqfoxext/fxx_action.hpp>
#include <aqfoxext/fxx_iconset.hpp>

#include <gwenhywfar/debug.h>



FXIMPLEMENT(FXXMenuContainer, FXObject, NULL, 0)



FXXMenuContainer::FXXMenuContainer()
:FXObject()
,typ(TYPE_UNKNOWN)
,action(NULL)
,parent(NULL)
,window(NULL)
,menuBar(NULL)
,menuTitle(NULL)
,menuCommand(NULL)
,menuCascade(NULL)
{
}



FXXMenuContainer::FXXMenuContainer(FXXMenuContainer *p,
                                   FXint t,
                                   const FXString &n,
                                   const FXString &text,
                                   FXIcon *ic,
                                   FXObject *tgt,
                                   FXSelector sel,
                                   FXuint opts)
:FXObject()
,typ(t)
,name(n)
,action(NULL)
,parent(p)
,window(NULL)
,menuBar(NULL)
,menuTitle(NULL)
,menuCommand(NULL)
,menuCascade(NULL)
{
  FXXMenuContainer *topContainer=NULL;

  FXASSERT(p);
  topContainer=p;
  while(topContainer->parent)
    topContainer=topContainer->parent;

  if (p) {
    if (p->typ!=TYPE_TITLE &&
        p->typ!=TYPE_MENUBAR &&
        p->typ!=TYPE_CASCADE) {
      fxerror("FXXMenuContainer::constructor: bad parent type %d\n",
              p->typ);
    }
    else {
      if (t==TYPE_COMMAND) {
        if (p->typ==TYPE_MENUBAR)
          menuCommand=new FXMenuCommand(p->menuBar,
                                        text, ic, tgt, sel, opts);
        else if (p->typ==TYPE_CASCADE) {
          menuCommand=new FXMenuCommand(p->menuCascade->getMenu(),
                                        text, ic, tgt, sel, opts);
        }
        else
          menuCommand=new FXMenuCommand(p->menuTitle->getMenu(),
                                        text, ic, tgt, sel, opts);
      }
      else if (t==TYPE_TITLE) {
        if (p->typ==TYPE_MENUBAR) {
          menuTitle=new FXMenuTitle(p->menuBar,
                                    text, ic,
                                    new FXMenuPane(topContainer->window)/*,
                                    opts*/);
        }
        else if (p->typ==TYPE_CASCADE) {
          menuTitle=new FXMenuTitle(p->menuCascade->getMenu(),
                                    text, ic,
                                    new FXMenuPane(topContainer->window),
                                    opts);
        }
        else
          menuTitle=new FXMenuTitle(p->menuTitle->getMenu(),
                                    text, ic,
                                    new FXMenuPane(topContainer->window),
                                    opts);
      }
      else if (t==TYPE_CASCADE) {
        if (p->typ==TYPE_MENUBAR) {
          menuCascade=new FXMenuCascade(p->menuBar,
                                        text, ic,
                                        new FXMenuPane(topContainer->window),
                                        opts);
        }
        else if (p->typ==TYPE_CASCADE) {
          menuCascade=new FXMenuCascade(p->menuCascade->getMenu(),
                                        text, ic,
                                        new FXMenuPane(topContainer->window),
                                        opts);
        }
        else {
          menuCascade=new FXMenuCascade(p->menuTitle->getMenu(),
                                        text, ic,
                                        new FXMenuPane(topContainer->window),
                                        opts);
        }
      }
      else {
        fxerror("FXXMenuContainer::constructor: bad type %d\n", t);
      }
    }
    p->addChild(this);
  }

}



FXXMenuContainer::FXXMenuContainer(FXComposite *p, FXuint opts)
:FXObject()
,typ(TYPE_MENUBAR)
,action(NULL)
,parent(NULL)
,window(NULL)
,menuBar(NULL)
,menuTitle(NULL)
,menuCommand(NULL)
,menuCascade(NULL)
{
  FXASSERT(p);
  menuBar=new FXMenuBar(p, opts);
  window=p;
}



FXXMenuContainer::~FXXMenuContainer() {
  FXint i;

  if (parent)
    parent->removeChild(this);
  if (action)
    action->removeMenuContainer(this);

  /* clear list of children */
  for (i=0; i<children.no(); i++) {
    children[i]->parent=NULL;
    delete children[i];
  }
  children.clear();

  delete menuCascade;
  menuCascade=(FXMenuCascade*) -1;
  delete menuCommand;
  menuCommand=(FXMenuCommand*) -1;
  delete menuTitle;
  menuTitle=(FXMenuTitle*) -1;
  delete menuBar;
  menuBar=(FXMenuBar*) -1;
}



FXXMenuContainer *FXXMenuContainer::getChildByName(const FXString &n) {
  FXint i;
  FXString s;

  s=n;
  s.lower();
  for (i=0; i<children.no(); i++) {
    if (children[i]->name.lower()==s)
      return children[i];
  }

  return NULL;
}



void FXXMenuContainer::addChild(FXXMenuContainer *ch) {
  children.append(ch);
}



void FXXMenuContainer::removeChild(FXXMenuContainer *ch) {
  FXint i;

  for (i=0; i<children.no(); i++) {
    if (children[i]==ch) {
      children.erase(i);
      break;
    }
  }
}



void FXXMenuContainer::enable() {
  switch(typ) {
  case TYPE_MENUBAR:
    menuBar->enable();
    break;
  case TYPE_COMMAND:
    menuCommand->enable();
    break;
  case TYPE_TITLE:
    menuTitle->enable();
    break;
  case TYPE_CASCADE:
    menuCascade->enable();
    break;
  default:
    break;
  }
}



void FXXMenuContainer::disable() {
  switch(typ) {
  case TYPE_MENUBAR:
    menuBar->disable();
    break;
  case TYPE_COMMAND:
    menuCommand->disable();
    break;
  case TYPE_TITLE:
    menuTitle->disable();
    break;
  case TYPE_CASCADE:
    menuCascade->disable();
    break;
  default:
    break;
  }
}



void FXXMenuContainer::create() {
  switch(typ) {
  case TYPE_MENUBAR:
    menuBar->create();
    break;
  case TYPE_COMMAND:
    menuCommand->create();
    break;
  case TYPE_TITLE:
    menuTitle->create();
    break;
  case TYPE_CASCADE:
    menuCascade->create();
    break;
  default:
    break;
  }
}



FXPopup *FXXMenuContainer::getMenuPane() {
  switch(typ) {
  case TYPE_MENUBAR:
    return NULL;
  case TYPE_COMMAND:
    return NULL;
  case TYPE_TITLE:
    return menuTitle->getMenu();
  case TYPE_CASCADE:
    return menuCascade->getMenu();
  default:
    return NULL;
  }
}



void FXXMenuContainer::insertSpacer(FXuint opts) {
  switch(typ) {
  case TYPE_MENUBAR:
    new FXSpring(menuBar, opts);
    break;
  case TYPE_COMMAND:
    break;
  case TYPE_TITLE:
    break;
  case TYPE_CASCADE:
    break;
  default:
    break;
  }
}



FXXMenuContainer *FXXMenuContainer::createMenuBar(FXComposite *w, FXuint opts) {
  return new FXXMenuContainer(w, opts);
}



FXXMenuContainer *FXXMenuContainer::createMenuCommand(FXXMenuContainer *p,
                                                      const FXString &name,
                                                      const FXString &text,
                                                      FXIcon *ic,
                                                      FXObject *tgt,
                                                      FXSelector sel,
                                                      FXuint opts) {
  FXASSERT(p);
  if (p->getChildByName(name))
    return NULL;
  return new FXXMenuContainer(p, TYPE_COMMAND, name, text, ic, tgt, sel, opts);
}



FXXMenuContainer *FXXMenuContainer::createMenuCommand(FXXMenuContainer *p,
                                                      FXXAction *action,
                                                      FXbool withText,
                                                      FXbool withIcon,
                                                      FXuint opts) {
  FXXMenuContainer *mc;

  FXASSERT(p);
  if (p->getChildByName(action->getName()))
    return NULL;
  mc=new FXXMenuContainer(p, TYPE_COMMAND,
                          action->getName(),
                          withText?action->getText():"",
                          withIcon?action->getIconSet()->getSmallIcon():NULL,
                          action->getTarget(),
                          action->getSelector());
  mc->action=action;
  return mc;
}



FXXMenuContainer *FXXMenuContainer::createMenuTitle(FXXMenuContainer *p,
                                                    const FXString &name,
                                                    const FXString &text,
                                                    FXIcon *ic,
                                                    FXuint opts) {
  FXASSERT(p);
  if (p->getChildByName(name))
    return NULL;
  return new FXXMenuContainer(p, TYPE_TITLE, name, text, ic, NULL, 0, opts);
}



FXXMenuContainer *FXXMenuContainer::createMenuCascade(FXXMenuContainer *p,
                                                      const FXString &name,
                                                      const FXString &text,
                                                      FXIcon *ic,
                                                      FXuint opts) {
  FXASSERT(p);
  if (p->getChildByName(name))
    return NULL;
  return new FXXMenuContainer(p, TYPE_CASCADE, name, text, ic, NULL, 0, opts);
}











